@@ -17,6 +17,6 @@ class CreatePromptImpulse():
     qualifier: str
     appendent: str
     utterance: dict
-    labellist: List[str]
+    labelList: List[str]
     version: str
 
