@@ -1,5 +1,4 @@
 
-from gaia_sdk.graphql.response.type.Intent import Intent
 
 from dataclasses import dataclass
 from typing import List
@@ -14,15 +13,15 @@ from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
 @dataclass
 class DeletedBehaviourImpulse:
     """
-    Impulse which indicates the resulf of a delete behaviour impulse
+    Impulse which indicates the result of a delete behaviour impulse
     """
     dictionary: dict
     @property
     def id(self) -> Uuid:
         return Uuid(self.dictionary.get("id"))
-    """
-    the behaviour instance
-    """
     @property
-    def data(self) -> Intent:
-        return Intent(self.dictionary.get("data"))
+    def identity_id(self) -> Uuid:
+        return Uuid(self.dictionary.get("identityId"))
+    @property
+    def reference(self) -> Uuid:
+        return Uuid(self.dictionary.get("reference"))
