@@ -1,14 +1,14 @@
-from typing import Callable, List
-
 import rx
 from rx.core.typing import Observable
+from typing import Callable, List
 
 from gaia_sdk.api.ISensorFunction import ISensorFunction
+from gaia_sdk.api.client_options import ClientOptions
 from gaia_sdk.api.rx.rx import mapQ, mapM, flat_mapM, flat_mapQ
-from gaia_sdk.graphql import QueryReq, QueryRes, RetrievalReq, ExperienceReq, KnowledgeReq, KnowledgeEdgeReq, \
+from gaia_sdk.graphql import QueryReq, QueryRes, RetrievalReq, ExperienceReq, KnowledgeReq, EdgeReq, \
     IntentReq, PromptReq, StatementReq, FulfilmentReq, CodeReq, BehaviourReq, IntrospectionReq, SkillIntrospectionReq, \
     PerceptionReq, PreservationReq, CreatedIntentImpulse, UpdatedIntentImpulse, DeletedIntentImpulse, RetrievalRes, \
-    ExperienceRes, KnowledgeEdgeRes, StatementRes, PromptRes, IntentRes, KnowledgeRes, FulfilmentRes, CodeRes, \
+    ExperienceRes, EdgeRes, StatementRes, PromptRes, IntentRes, KnowledgeRes, FulfilmentRes, CodeRes, \
     BehaviourRes, IntrospectionRes, SkillIntrospectionRes, PreservationRes, PerceptionRes, PerceivedImpulse, \
     MutationReq, MutationRes, PerceiveDataImpulse, PerceiveActionImpulse, DeleteIntentImpulse, \
     UpdateIntentImpulse, CreateIntentImpulse, CreatePromptImpulse, UpdatePromptImpulse, DeletePromptImpulse, \
@@ -18,12 +18,13 @@ from gaia_sdk.graphql import QueryReq, QueryRes, RetrievalReq, ExperienceReq, Kn
     UpdateFulfilmentImpulse, DeleteFulfilmentImpulse, CreatedFulfilmentImpulse, UpdatedFulfilmentImpulse, \
     DeletedFulfilmentImpulse, CreateBehaviourImpulse, UpdateBehaviourImpulse, DeleteBehaviourImpulse, \
     CreatedBehaviourImpulse, UpdatedBehaviourImpulse, DeletedBehaviourImpulse, CreateCodeImpulse, UpdateCodeImpulse, \
-    DeleteCodeImpulse, CreatedCodeImpulse, UpdatedCodeImpulse, DeletedCodeImpulse
+    DeleteCodeImpulse, CreatedCodeImpulse, UpdatedCodeImpulse, DeletedCodeImpulse, CreateEdgeImpulse, \
+    DeleteEdgeImpulse, CreatedEdgeImpulse, DeletedEdgeImpulse
 from gaia_sdk.graphql.GaiaClient import GaiaClient
 from gaia_sdk.graphql.GaiaRequest import GaiaRequest
 from gaia_sdk.http.HttpTransporter import HttpTransporter
-from gaia_sdk.api.client_options import ClientOptions
 
+Uuid = str
 
 class HttpSensorFunction(ISensorFunction):
 
@@ -53,17 +54,26 @@ class HttpSensorFunction(ISensorFunction):
         observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
         return mapQ(observable, query_res)
 
-    def retrieve_knowledge_edge(self, config: Callable[[KnowledgeEdgeReq], None]) -> Observable[KnowledgeEdgeRes]:
-        knowledge_req: Callable[[KnowledgeReq], None] = lambda x: x.edges(config)
+    def retrieve_edges(self, source: Uuid, config: Callable[[EdgeReq], None]) -> Observable[EdgeRes]:
+        knowledge_req: Callable[[KnowledgeReq], None] = lambda x: x.edges(source, config)
         retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(knowledge_req)
         query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
-        query_res: Callable[[QueryRes], KnowledgeEdgeRes] = lambda x: x.retrieve.knowledge.edges
+        query_res: Callable[[QueryRes], EdgeRes] = lambda x: x.retrieve.knowledge.edges
 
         observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
         return flat_mapQ(observable, query_res)
 
-    def retrieve_intents(self, config: Callable[[IntentReq], None]) -> Observable[IntentRes]:
-        intents_req: Callable[[IntentReq], None] = lambda x: x.intents(config)
+    def retrieve_edge(self, source: Uuid, target: Uuid, config: Callable[[EdgeReq], None]) -> Observable[EdgeRes]:
+        knowledge_req: Callable[[KnowledgeReq], None] = lambda x: x.edge(source, target, config)
+        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(knowledge_req)
+        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
+        query_res: Callable[[QueryRes], EdgeRes] = lambda x: x.retrieve.knowledge.edge
+
+        observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
+        return mapQ(observable, query_res)
+
+    def retrieve_intents(self, identityId: Uuid, config: Callable[[IntentReq], None]) -> Observable[IntentRes]:
+        intents_req: Callable[[IntentReq], None] = lambda x: x.intents(identityId, config)
         retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(intents_req)
         query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
         query_res: Callable[[QueryRes], IntentRes] = lambda x: x.retrieve.knowledge.intents
@@ -71,8 +81,17 @@ class HttpSensorFunction(ISensorFunction):
         observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
         return flat_mapQ(observable, query_res)
 
-    def retrieve_prompts(self, config: Callable[[PromptReq], None]) -> Observable[PromptRes]:
-        prompts_req: Callable[[PromptReq], None] = lambda x: x.prompts(config)
+    def retrieve_intent(self, identityId: Uuid, reference: Uuid, config: Callable[[IntentReq], None]) -> Observable[IntentRes]:
+        intents_req: Callable[[IntentReq], None] = lambda x: x.intent(identityId, reference, config)
+        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(intents_req)
+        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
+        query_res: Callable[[QueryRes], IntentRes] = lambda x: x.retrieve.knowledge.intent
+
+        observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
+        return mapQ(observable, query_res)
+
+    def retrieve_prompts(self, identityId: Uuid, config: Callable[[PromptReq], None]) -> Observable[PromptRes]:
+        prompts_req: Callable[[PromptReq], None] = lambda x: x.prompts(identityId, config)
         retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(prompts_req)
         query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
         query_res: Callable[[QueryRes], PromptRes] = lambda x: x.retrieve.knowledge.prompts
@@ -80,8 +99,17 @@ class HttpSensorFunction(ISensorFunction):
         observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
         return flat_mapQ(observable, query_res)
 
-    def retrieve_statements(self, config: Callable[[StatementReq], None]) -> Observable[StatementRes]:
-        statement_req: Callable[[StatementReq], None] = lambda x: x.statements(config)
+    def retrieve_prompt(self, identityId: Uuid, reference: Uuid, config: Callable[[PromptReq], None]) -> Observable[PromptRes]:
+        prompts_req: Callable[[PromptReq], None] = lambda x: x.prompt(identityId, reference, config)
+        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(prompts_req)
+        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
+        query_res: Callable[[QueryRes], PromptRes] = lambda x: x.retrieve.knowledge.prompt
+
+        observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
+        return mapQ(observable, query_res)
+
+    def retrieve_statements(self,identityId: Uuid, config: Callable[[StatementReq], None]) -> Observable[StatementRes]:
+        statement_req: Callable[[StatementReq], None] = lambda x: x.statements(identityId, config)
         retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(statement_req)
         query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
         query_res: Callable[[QueryRes], StatementRes] = lambda x: x.retrieve.knowledge.statements
@@ -89,8 +117,17 @@ class HttpSensorFunction(ISensorFunction):
         observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
         return flat_mapQ(observable, query_res)
 
-    def retrieve_fulfilments(self, config: Callable[[FulfilmentReq], None]) -> Observable[FulfilmentRes]:
-        fulfilment_req: Callable[[FulfilmentReq], None] = lambda x: x.fulfilments(config)
+    def retrieve_statement(self, identityId: Uuid, reference: Uuid, config: Callable[[StatementReq], None]) -> Observable[StatementRes]:
+        statement_req: Callable[[StatementReq], None] = lambda x: x.statement(identityId, reference, config)
+        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(statement_req)
+        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
+        query_res: Callable[[QueryRes], StatementRes] = lambda x: x.retrieve.knowledge.statement
+
+        observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
+        return mapQ(observable, query_res)
+
+    def retrieve_fulfilments(self, identityId: Uuid, config: Callable[[FulfilmentReq], None]) -> Observable[FulfilmentRes]:
+        fulfilment_req: Callable[[FulfilmentReq], None] = lambda x: x.fulfilments(identityId, config)
         retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(fulfilment_req)
         query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
         query_res: Callable[[QueryRes], FulfilmentRes] = lambda x: x.retrieve.knowledge.fulfilments
@@ -98,8 +135,17 @@ class HttpSensorFunction(ISensorFunction):
         observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
         return flat_mapQ(observable, query_res)
 
-    def retrieve_codes(self, config: Callable[[CodeReq], None]) -> Observable[CodeRes]:
-        code_req: Callable[[CodeReq], None] = lambda x: x.codes(config)
+    def retrieve_fulfilment(self, identityId: Uuid, reference: Uuid, config: Callable[[FulfilmentReq], None]) -> Observable[FulfilmentRes]:
+        fulfilment_req: Callable[[FulfilmentReq], None] = lambda x: x.fulfilment(identityId, reference, config)
+        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(fulfilment_req)
+        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
+        query_res: Callable[[QueryRes], FulfilmentRes] = lambda x: x.retrieve.knowledge.fulfilment
+
+        observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
+        return mapQ(observable, query_res)
+
+    def retrieve_codes(self, identityId: Uuid, config: Callable[[CodeReq], None]) -> Observable[CodeRes]:
+        code_req: Callable[[CodeReq], None] = lambda x: x.codes(identityId, config)
         retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(code_req)
         query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
         query_res: Callable[[QueryRes], CodeRes] = lambda x: x.retrieve.knowledge.codes
@@ -107,8 +153,17 @@ class HttpSensorFunction(ISensorFunction):
         observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
         return flat_mapQ(observable, query_res)
 
-    def retrieve_behaviour(self, config: Callable[[BehaviourReq], None]) -> Observable[BehaviourRes]:
-        behaviour_req: Callable[[BehaviourReq], None] = lambda x: x.behaviours(config)
+    def retrieve_code(self, identityId: Uuid, reference: Uuid, config: Callable[[CodeReq], None]) -> Observable[CodeRes]:
+        code_req: Callable[[CodeReq], None] = lambda x: x.code(identityId, reference, config)
+        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(code_req)
+        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
+        query_res: Callable[[QueryRes], CodeRes] = lambda x: x.retrieve.knowledge.code
+
+        observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
+        return mapQ(observable, query_res)
+
+    def retrieve_behaviours(self, identityId: Uuid, config: Callable[[BehaviourReq], None]) -> Observable[BehaviourRes]:
+        behaviour_req: Callable[[BehaviourReq], None] = lambda x: x.behaviours(identityId, config)
         retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(behaviour_req)
         query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
         query_res: Callable[[QueryRes], BehaviourRes] = lambda x: x.retrieve.knowledge.behaviours
@@ -116,6 +171,15 @@ class HttpSensorFunction(ISensorFunction):
         observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
         return flat_mapQ(observable, query_res)
 
+    def retrieve_behaviour(self, identityId: Uuid, reference: Uuid, config: Callable[[BehaviourReq], None]) -> Observable[BehaviourRes]:
+        behaviour_req: Callable[[BehaviourReq], None] = lambda x: x.behaviour(identityId, reference, config)
+        retrieval_req: Callable[[RetrievalReq], None] = lambda x: x.knowledge(behaviour_req)
+        query_req: Callable[[QueryReq], None] = lambda x: x.retrieve(retrieval_req)
+        query_res: Callable[[QueryRes], BehaviourRes] = lambda x: x.retrieve.knowledge.behaviour
+
+        observable = rx.of(self.client.query(GaiaRequest.query(query_req)))
+        return mapQ(observable, query_res)
+
     def introspect(self, config: Callable[[IntrospectionReq], None]) -> Observable[IntrospectionRes]:
         query_req: Callable[[QueryReq], None] = lambda x: x.introspect(config)
         query_res: Callable[[QueryRes], IntrospectionRes] = lambda x: x.introspect
@@ -300,6 +364,24 @@ class HttpSensorFunction(ISensorFunction):
         observable = rx.of(self.client.mutation(GaiaRequest.mutation(mutation_req)))
         return flat_mapM(observable, mutation_res)
 
+    def preserve_create_edges(self, impulses: List[CreateEdgeImpulse]) -> Observable[CreatedEdgeImpulse]:
+        edge_req = lambda x: x.id()
+        create_req: Callable[[PreservationReq], None] = lambda x: x.create(lambda e: e.edges(impulses, edge_req))
+        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(create_req)
+        mutation_res: Callable[[MutationRes], CreatedEdgeImpulse] = lambda x: x.preserve.create.edges
+
+        observable = rx.of(self.client.mutation(GaiaRequest.mutation(mutation_req)))
+        return flat_mapM(observable, mutation_res)
+
+    def preserve_delete_edges(self, impulses: List[DeleteEdgeImpulse]) -> Observable[DeletedEdgeImpulse]:
+        edge_req = lambda x: x.id()
+        delete_edges: Callable[[PreservationReq], None] = lambda x: x.delete(lambda e: e.edges(impulses, edge_req))
+        mutation_req: Callable[[MutationReq], None] = lambda x: x.preserve(delete_edges)
+        mutation_res: Callable[[MutationRes], DeletedEdgeImpulse] = lambda x: x.preserve.delete.edges
+
+        observable = rx.of(self.client.mutation(GaiaRequest.mutation(mutation_req)))
+        return flat_mapM(observable, mutation_res)
+
     def perceive(self, config: Callable[[PerceptionReq], None]) -> Observable[PerceptionRes]:
         mutation_req: Callable[[MutationReq], None] = lambda x: x.perceive(config)
         mutation_res: Callable[[MutationRes], PerceptionRes] = lambda x: x.perceive
